﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace QR_PIXEL_Lesen
{
    class RS_Decode
    {
        public string kommentar;
        private string irreduziblesPolynom;
        private int irreduziblesPolynom_Dez;


        private int[] log2Feld = new int[256];
        private int[] pot2Feld = new int[256];
        public int[] invFeld = new int[256];

        int[] generator = new int[256];

        int[] //nachrichten, 
              //nachrichtenECC, 
              originalBytes;  // wie beim Aufruf der Decode-Routine erhalten

        int[] lesecode, originalLesecode;
        int fehler = -1, fehlerAnzahl = 0;
        int[] endeCode;

        int[,] mat;
        int[] sigma = new int[255];
        int[] fehlerstelle = new int[255];

        int nachrichtenWortLänge = 0;
        int codeWortlänge = 0;
        int korrekturWortLänge = 0;


        int[] syndrom = new int[20];
        bool fehlerhaft = false;
        bool ausnahme1Fb2KB = false;
        int ausnahme1Fb2KB_stelle = -1;
        int ausnahme1Fb2KB_wert = -1;

        private string arrayAnzeige(int[] a)
        {
            string s = "";
            for (int i=a.Length-1;i>=0;i--)
            {
                s = s + (a[i].ToString() + "*x^" + i.ToString()).PadLeft(9);
                if (i > 0) s = s + " + ";
            }
            return s;
        }

        private void Vorbereitung()
        {
            // Vorbereitung
            for (int i = 1; i <= 255; i++)
            {
                int wert = xHyR(2, i); // jetzt R_ekursiv .... alt: xHoch_y(2, i) ohne Rekursion;
                log2Feld[wert] = i;
                pot2Feld[i] = wert;
            }
            for (int i = 1; i <= 255; i++)
            {
                invFeld[i] = xHyR(2, 255 - log2Feld[i]);
            }
        }

        public RS_Decode(string irreduziblesPoly)
        {
            irreduziblesPolynom = irreduziblesPoly;
            irreduziblesPolynom_Dez = Convert.ToInt16(irreduziblesPolynom, 2);  
                                    
            Vorbereitung();
        }

        public RS_Decode()
        {
            irreduziblesPolynom_Dez = 285;  
            // QR-Code bei Data Matrix normal 301 (binär 100101101)
            // QR-Code: normal 285 (binär 100011101)

            Vorbereitung();
        }


        public int xHyR(int x, int y)
        {
            int erg = 0;
            if (y >= 2)
            {
                int h = xHyR(x, y / 2);
                erg = multiplikation(h, h, false);
                if (y % 2 != 0)
                {
                    erg = multiplikation(erg, x, false);
                }
            }
            else
            {
                if (y == 1) erg = x;
                else erg = 1;
            }
            return erg;
        }

        private int[] polynomMultiplikationBinär(int[] a, int[] b, int moduloWert) // nur relevant bei moduloWert>1
        {
            // 3x^5 + 12x^3 - 5x^2 +  x - 99 = 99 + x - 5x^2 + 12x^3 + 3x^5  --->  a[0]=99, a[1]=1, ... -5, 12, 0, 3
            // c = a * b;
            int[] c = new int[a.Length + b.Length];
            for (int i = 0; i < c.Length; i++) c[i] = 0;

            for (int i = 0; i < b.Length; i++)
                for (int j = 0; j < a.Length; j++)
                {
                    int wert = 0;
                    if ((a[j] * b[i]) != 0)
                    {
                        int hz1 = log2Feld[a[j]], hz2 = log2Feld[b[i]];
                        int hz = (hz1 + hz2) % 255;
                        wert = xHyR(2, hz);
                    }
                    c[i + j] = c[i + j] ^ wert;  // a[j] * b[i];
                }
            if (moduloWert > 1) for (int i = 0; i < c.Length; i++) c[i] = c[i] % moduloWert;
            return c;
        }

        public int multiplikation(int w1, int w2, bool anzeige)
        {
            // siehe     h t t p s://en.wikipedia.org/wiki/Finite_field_arithmetic
            int p = 0; /* Produkt der Multiplikation */
            int a = w1;
            int b = w2;
            while ((a != 0) && (b != 0))
            {
                if ((b % 2) > 0) // falls b gerade ist ... Addition von a zum Produkt 
                                 // Endprodukt = Summe aller  a's ... bei geraden b's 
                    p ^= a;      // wir bewegen uns im GF(2^m) --> Addition mit XOR 
                if (a >= 128)    // GF modulo: if a >= 128, then it will overflow when shifted left, so reduce 
                    a = (a << 1) ^ irreduziblesPolynom_Dez;  //  285; 
                                                             // irreversibles Polynom      1 + x ^ 2 + x ^ 3 + x ^ 4 + x ^ 8;     100011101 -- > 285
                                                             // 0x11b; /* XOR with the primitive polynomial x^8 + x^4 + x^3 + x + 1 (0b100011011) --> 
                                                             //            – you can change it but it must be irreducible */
                else
                    a <<= 1; // entspricht  a = a * 2 
                b >>= 1; // entspricht  b = b / 2 
            }

            return p;
        }

        private int[] erstelleGeneratorpolynom(int k)
        {
            int[] gen;
            int[] a = { 1, 1 };
            int[] b = { 2, 1 };
            int[] c;
            int letztWert = 0;

            if (k > 1)
            {
                c = polynomMultiplikationBinär(a, b, 256); // polynomMultiplikationGenerator(a, b);
                letztWert = 0;
                for (int j = c.Length - 1; j >= 0; j--) if (c[j] > 0) { letztWert = j; j = -1; }
                for (int j = 0; j <= letztWert; j++)
                {
                    if (j >= a.Length) { Array.Resize(ref a, a.Length + 1); }
                    a[j] = c[j];
                }
                //richTextBox4.AppendText("\n (1+x)(2+x) ... ");
                //for (int j = 0; j < a.Length; j++) richTextBox4.AppendText("  " + a[j].ToString());
            }

            if (k > 2)
            {
                for (int i = 1; i <= k - 2; i++)
                {
                    b[0] = multiplikation(b[0], 2, false);
                    b[1] = 1;
                    //a = polynomMultiplikationGenerator(a, b);
                    c = polynomMultiplikationBinär(a, b, 256); // polynomMultiplikationGenerator(a, b);
                    letztWert = 0;
                    for (int j = c.Length - 1; j >= 0; j--) if (c[j] > 0) { letztWert = j; j = -1; }
                    for (int j = 0; j <= letztWert; j++)
                    {
                        if (j >= a.Length) Array.Resize(ref a, a.Length + 1);
                        a[j] = c[j];
                    }

                    //richTextBox4.AppendText("  <---| * (" + b[0].ToString() + " + x)\n");
                    //for (int j = 0; j < a.Length; j++) richTextBox4.AppendText("  " + a[j].ToString());

                }
            }
            //bool start = false;
            //string poly = "";
            letztWert = 0;
            for (int i = a.Length - 1; i >= 0; i--) if (a[i] > 0) { letztWert = i; i = -1; }
            Array.Resize(ref a, letztWert + 1);
            gen = a;
            kommentar = kommentar + "\nGenertorpolynom:\n" + arrayAnzeige(gen);
            
            return gen;
        }

        private int[] polynomDivisionRest(int[] aOrg, int[] b)
        {
            int[] a = new int[aOrg.Length];
            for (int i = 0; i < a.Length; i++) a[i] = aOrg[i];

            int hoechstBit = 0;
            int[] h = new int[a.Length];

            for (int i = a.Length - 1; i >= 0; i--) if (a[i] > 0) { hoechstBit = i; i = -1; }

            while (hoechstBit >= b.Length - 1)
            {
                int diff = hoechstBit - (b.Length - 1);
                for (int i = b.Length - 1; i >= 0; i--) h[i + diff] = multiplikation(a[hoechstBit], b[i], false);
                for (int i = diff - 1; i >= 0; i--) h[i] = 0; // rechts mit Nullen auffüllen

                for (int i = hoechstBit; i >= 0; i--) a[i] = a[i] ^ h[i]; // XOR wegen "modulo 2" -Rechnung;   ZwischenRest in a
                for (int i = a.Length - 1; i >= 0; i--) if (a[i] > 0) { hoechstBit = i; i = -1; }
            }
            return a;
        }

        public int[] IntArrayUmdrehen(int[] a)
        {
            int[] aHilf = new int[a.Length];
            for (int i = 0; i < a.Length; i++) aHilf[a.Length - 1 - i] = a[i];
            return aHilf;
        }


        public int[] stringInIntArray(string s, char trenner)
        {
            int[] a = new int[1]; ;
            if (s != "")
            {
                string[] datenStrings = s.Split(trenner); // Aufspalten mit Leerzeichen-Trenner
                Array.Resize(ref a, datenStrings.Length);
                //a = new int[datenStrings.Length];  // alle Bytes als Strings
                int zw = 0, h1 = -1;
                for (int i = 0; i < datenStrings.Length; i++)
                {
                    if (datenStrings[i].Trim() != "")
                    {
                        try { h1 = Convert.ToInt16(datenStrings[i].Trim()); } catch { h1 = -1; }
                        if (h1 >= 0)
                        {
                            a[zw] = h1;
                            zw++;
                        }
                    }
                }
                Array.Resize(ref a, zw);
            }
            return a;
        }






        private void bildeArrayLesen_Click(string datenBytes, int anzahlECC)
        {
            
            string s = datenBytes.Trim(); ;
            if (s != "")
            {
                kommentar = kommentar + "\n\nEinlesen der Daten:\n ";
                
                // Nachrichten in int-Array umwandeln:
                korrekturWortLänge = anzahlECC;

                string[] datenStrings = s.Split(' '); // Aufspalten mit Leerzeichen-Trenner
                int[] a = new int[datenStrings.Length];  // alle Bytes als Strings
                int zw = 0, h1 = -1;
                for (int i = 0; i < datenStrings.Length; i++)
                {
                    if (datenStrings[i].Trim() != "")
                    {
                        try { h1 = Convert.ToInt16(datenStrings[i].Trim()); } catch { h1 = -1; }
                        if (h1 >= 0)
                        {
                            a[zw] = h1; 
                            kommentar = kommentar + a[zw].ToString("##0").PadLeft(4);
                            zw++;
                        }
                    }
                }

                Array.Resize(ref a, zw);
                codeWortlänge = zw;
                nachrichtenWortLänge = codeWortlänge - korrekturWortLänge;
                // Nachrichten erweitern auf GerneratorpolynomLänge und Reihenfolge umdrehen
                int origLaenge = a.Length;

                int[] aHilf = new int[a.Length];
                for (int i = 0; i < a.Length; i++) aHilf[a.Length - 1 - i] = a[i];
                for (int i = 0; i < a.Length; i++) a[i] = aHilf[i];

                originalBytes = new int[a.Length];
                for (int i = 0; i < a.Length; i++)
                {
                    originalBytes[i] = a[i];
                    
                }


                lesecode = a;  // wird weiter verwendet

                originalLesecode = a;
                kommentar = kommentar + "\n";
            }
            else kommentar = kommentar + "\nBitte Code eingeben ...";
        }




        private int auswertung(int[] a, int x)   // zuerst für syndromB_Click()
        {
            int wert = 0;
            for (int i = a.Length - 1; i >= 0; i--)
            {
                int potwert = xHyR(x, i);
                int addWert = multiplikation(a[i], potwert, false);
                wert = wert ^ addWert;
                //kommentar = kommentar + a[i].ToString().PadLeft(3) + "/" + potwert.ToString().PadLeft(3)+", ";
            }

            //kommentar = kommentar +" -> " + wert.ToString().PadLeft(4) + "\n";
            return wert;
        }

        private void syndromB_Click()
        {
            int k = korrekturWortLänge;
            kommentar = kommentar + "\nSyndromberechnung:\nbei " + k.ToString() + " ECC-Bytes\n";
            fehlerhaft = false;
            if (k > 0)
            {
                Array.Resize(ref syndrom, k + 1);
                for (int i = 0; i < k; i++)
                {
                    int po = xHyR(2, i);
                    syndrom[i] = auswertung(lesecode, po); // für 2, 4, 8, ...
                    if (syndrom[i] > 0) fehlerhaft = true;
                    kommentar = kommentar + "  2^" + i.ToString().PadLeft(3) + " = "  + po.ToString().PadLeft(3) + " --> " + syndrom[i].ToString().PadLeft(5)+"\n";
                }
            }
            if (fehlerhaft != true)
            {
                fehlerAnzahl = 0;
                kommentar = kommentar + "\nmit Fehlern!!\n";
            }
        }


        private int Determinante2(int[,] matrix)
        {
            int n = mat.GetLength(0);
            int result = 1;
            kommentar = kommentar + "\nMatrix:\n"; // mat[ze, sp].ToString().PadLeft(4);
            int[,] matrixNeu = new int[n, n];
            for (int i = 0; i < n; i++)
                for (int j = 0; j < n; j++) matrixNeu[i, j] = matrix[i, j];

            for (int i = 0; i < n; i++)
            {
                for (int j = 0; j < n; j++)
                    kommentar = kommentar + matrixNeu[i, j].ToString().PadLeft(4);
                kommentar = kommentar + "\n";
            }
            kommentar = kommentar + "\n";

            for (int i = 0; i < n; i++)
            {
                if (matrixNeu[i, i] == 0)
                {
                    for (int zeile = i; zeile < n; zeile++)
                    {
                        if (matrixNeu[i, zeile] != 0) // Eine gefunden mit Erstzahl <> 0
                        { // Zeile tauschen
                            for (int k = 0; k < n; k++)
                            {
                                int tmp = matrixNeu[k, i];
                                matrixNeu[k, i] = matrixNeu[k, zeile];
                                matrixNeu[k, zeile] = tmp;
                            }

                            zeile = n;
                        }

                    }
                } // jetzt steht links oben sicher Zahl <> 0 !!

                
                if (matrixNeu[i, i] == 0) continue;

                int faktor1 = invFeld[matrixNeu[i, i]];  // vorn alle auf gleichen Wert bringen 
                for (int j = i + 1; j < n; j++)
                {
                    // 0 er erzeugen durch Addition
                    if (matrixNeu[j, i] > 0)
                    {
                        int faktor2 = invFeld[matrixNeu[j, i]]; // 
                        for (int k = 0; k < n; k++)
                        {
                            matrixNeu[j, k] = multiplikation(faktor2, matrixNeu[j, k], false) ^ multiplikation(faktor1, matrixNeu[i, k], false);
                        }
                    }
                    //result = multiplikation(multiplikation(faktor1,invFeld[faktor2],false), result, false);
                }
            }
            kommentar = kommentar + "\numgeformte Matrix:\n";
                for (int i = 0; i < n; i++)
                { 
                    for (int j = 0; j < n; j++)
                        kommentar = kommentar + matrixNeu[i, j].ToString().PadLeft(4);
                    kommentar = kommentar + "\n";
                }
                kommentar = kommentar + "\n";


            for (int i = n - 1; i > 0; i--)
            {
                if (matrixNeu[i, i] != 0)
                {
                    fehlerAnzahl = i + 1;
                    i = 0;
                }
            }

            for (int i = 0; i < n; i++)
            {
                result = multiplikation(result, matrixNeu[i, i], false);
            }

            kommentar = kommentar + "\nDeterminanten-Ergebnis: " + result.ToString() + 
                "         Fehleranzahl: " + fehlerAnzahl.ToString() + "\n";
            return result;
        }




        private void determinaten_Click()
        {
            int k = korrekturWortLänge/2; // r-Wert ... max. Zahl die korr. werden kann
            int korr = korrekturWortLänge; // Korrektur-Wert
            kommentar = kommentar + "\nDeterminantenbestimmung (mit k="+k.ToString()+")\n";

            kommentar = kommentar + "\nMatrix\n";
            fehlerAnzahl = 0;
            if (k > 0)
            {
                fehler = -1;
                if (k > 1)
                {
                    for (int i = k; i >= 2; i--)
                    {
                        mat = new int[i, i];
                        for (int sp = i - 1; sp >= 0; sp--)
                        { 
                            for (int ze = 0; ze < i; ze++)
                            {
                                mat[ze, sp] = syndrom[(ze + (i - 1 - sp)) % korr]; // + 1];
                            }
                        }
                        
                        int determinante_h = 0;
                        determinante_h = Determinante2(mat); // neues Verfahren ... schneller!!

                        if (determinante_h != 0)
                        {
                            fehler = determinante_h;
                            fehlerAnzahl = i;
                            kommentar = kommentar + "\nDeterminante: " + determinante_h.ToString().PadLeft(4) +
                                "\nFehlerzahl: " + i.ToString().PadLeft(4) + "\n";

                            i = 0;
                        }
                        if (fehlerAnzahl > 0) { fehler = 1; i = 0; }
                    }

                    // Korrektur !!  8.1.2018
                    if (fehlerAnzahl > 1)
                    {
                        int hilf = mat.GetLength(0);
                        int[,] matHilf = new int[fehlerAnzahl, fehlerAnzahl];
                        for (int i1 = 0; i1 < fehlerAnzahl; i1++)
                            for (int j1 = 0; j1 < fehlerAnzahl; j1++) matHilf[i1, j1] = mat[i1, j1 + hilf - fehlerAnzahl];

                        mat = new int[fehlerAnzahl, fehlerAnzahl];
                        for (int i1 = 0; i1 < fehlerAnzahl; i1++)
                            for (int j1 = 0; j1 < fehlerAnzahl; j1++) mat[i1, j1] = matHilf[i1, j1];

                        kommentar = kommentar + "\nRelevante Matrix:\n";
                            for (int i1 = 0; i1 < fehlerAnzahl; i1++)
                            {
                                for (int j1 = 0; j1 < fehlerAnzahl; j1++)
                                    kommentar = kommentar + mat[i1, j1].ToString().PadLeft(4);
                                kommentar = kommentar + "\n";
                            }
                            kommentar = kommentar + "\n";
                    }

                    if (fehlerAnzahl == 1)
                    {
                        mat = new int[1, 1]; // damit erkennt man, dass es nicht "normal" weitergeht
                    }
                    // Korrekturende

                }
                else // k=1;  max. ein Fehler!
                {
                    mat = new int[1, 1]; // damit erkennt man, dass es nicht "normal" weitergeht

                }

                if (fehler < 0)
                {
                    if (syndrom[1] != 0)
                    {
                        fehler = 1;
                        fehlerAnzahl = 1;
                        mat = new int[1, 1]; // damit erkennt man, dass es nicht "normal" weitergeht

                    }
                }

            }

        }




        private int[] LGS(int[,] matrix, int[] konstante) // erg[1] ist erste Variable !!  erg[0] bleibt frei
        {
            kommentar = kommentar + "\nLSG am Anfang:\n";
            for (int i1 = 0; i1 < matrix.GetLength(0); i1++)
            {
                for (int j1 = 0; j1 < matrix.GetLength(1); j1++)
                    kommentar = kommentar + matrix[i1, j1].ToString().PadLeft(4);
                kommentar = kommentar + " - " + konstante[i1].ToString().PadLeft(4) + "\n";
            }
            kommentar = kommentar + "\n";




            int[] erg = new int[matrix.GetLength(0) + 1];
            int anz = matrix.GetLength(0);

            if (matrix[0, 0] == 0)  // vertauschen, so dass links oben Zahl <> 0 steht
            {
                int h = 0;
                for (int i = 0; i < anz; i++)
                {
                    if (matrix[i, 0] != 0)
                    {
                        for (int j = 0; j < anz; j++)
                        {
                            h = matrix[0, j];
                            matrix[0, j] = matrix[i, j];
                            matrix[i, j] = h;
                        }
                        h = konstante[0]; konstante[0] = konstante[i]; konstante[i] = h;
                        i = anz; // Ende
                    }
                }
            }
            

            for (int ver = 0; ver < anz - 1; ver++) 
            {
                if (matrix[ver, ver] == 0) // oben links muss Zahl <> 0 stehen!
                {
                    int h = 0;
                    for (int i = ver + 1; i < anz; i++)
                    {
                        if (matrix[i, ver] != 0) // Zeile suchen (1. Zahl <> 0) und dann tauschen
                        {
                            for (int j = ver; j < anz; j++) 
                            {
                                h = matrix[ver, j];
                                matrix[ver, j] = matrix[i, j];
                                matrix[i, j] = h;
                            }
                            h = konstante[ver]; konstante[ver] = konstante[i]; konstante[i] = h;
                            i = anz; // Ende
                        }
                    }
                }





                for (int ze = ver; ze < anz; ze++) // ze 0 .. 1
                {
                    if (matrix[ze, ver] > 0)
                    {
                        int inv = invFeld[matrix[ze, ver]];     
                        // RT.AppendText("  inverses El. von " + matrix[ze, ver].ToString("000") + 
                        //              " ist " + inv.ToString("000") + "\n");

                        for (int sp = ver; sp < anz; sp++) //  vorn auf 1 bringen
                            matrix[ze, sp] = multiplikation(matrix[ze, sp], inv, false);
                        konstante[ze] = multiplikation(konstante[ze], inv, false);
                    }
                }

                for (int ze = ver + 1; ze < anz; ze++) // subtrahieren ... vorn 1 oben sonst 0
                {
                    if (matrix[ze, ver] > 0)
                    {
                        for (int sp = ver; sp < anz; sp++) matrix[ze, sp] = matrix[ver, sp] ^ matrix[ze, sp];
                        konstante[ze] = konstante[ze] ^ konstante[ver];
                    }
                }


            }  // Ende von for (int ver = 0; ver < anz - 1; ver++) --> Dreiecksform

            
            for (int i1 = 0; i1 < fehlerAnzahl; i1++)
            {
                for (int j1 = 0; j1 < fehlerAnzahl; j1++)
                    kommentar = kommentar + matrix[i1, j1].ToString().PadLeft(4);
                kommentar = kommentar + " - " + konstante[i1].ToString().PadLeft(4) + "\n";
            }
            kommentar = kommentar + "\n";

            erg[anz] = multiplikation(konstante[anz - 1], invFeld[matrix[anz - 1, anz - 1]], false);
            if (anz > 1)
            {
                for (int ze = anz - 2; ze >= 0; ze--)
                {
                    int wert = 0;
                    for (int sp = ze + 1; sp <= anz - 1; sp++) wert = wert ^ multiplikation(matrix[ze, sp], erg[sp + 1], false);
                    erg[ze + 1] = multiplikation(konstante[ze] ^ wert, invFeld[matrix[ze, ze]], false);
                }
            }

            kommentar = kommentar + "\n Ergebnis: \n";
            for (int i1 = 0; i1 < erg.Length; i1++)
            {
                kommentar = kommentar + erg[i1].ToString().PadLeft(4);
            }
            kommentar = kommentar + "\n";

            return erg;
        }





        private void LGS_sigma_Click()
        {
            int anz = mat.GetLength(0); // relevante Matrix wird hier verwendet
            /**/
            if (anz > 1)
            {
                if (anz == mat.GetLength(1)) // quadratisch links ... für die Variablen sigma_1 bis sigma_anz
                {
                    // rechte Seite mit Konstanten syndrom(anz+1) ...
                    int[] kon = new int[anz];
                    int[,] matUr = new int[anz, anz];
                    for (int i = 0; i < anz; i++) // Kopie von mat
                    {
                        for (int j = 0; j < anz; j++)
                            matUr[i, j] = mat[i, j];

                    }

                    for (int i = 0; i < anz; i++) kon[i] = syndrom[anz + i]; // + 1];
                    int[] konUr = new int[anz];
                    for (int i = 0; i < anz; i++) konUr[i] = kon[i]; // Kopie von kon


                    sigma = LGS(mat, kon);
                   
                }

            }
            else // nur ein Fehler (bei 2 oder mehr KorrBytes)
            {
                int a = -1, b = -1, Hochzahl = -1, divWert = -1;
                if (anz == 1)
                {
                    kommentar = kommentar + "\nNur ein Fehler!\n";
                    if (syndrom[1] != syndrom[0])
                    {
                        a = syndrom[1];
                        b = invFeld[syndrom[0]];
                        divWert = multiplikation(a, b, false);
                        Hochzahl = log2Feld[divWert];
                        kommentar = kommentar + "s(2) = " + a.ToString() + ";   Inv(s(1)) = " + b.ToString() + 
                            "     Hochzahl = " + Hochzahl.ToString();
                    }
                    else
                    {
                        Hochzahl = 0;
                    }

                    int f = 0;
                    if (Hochzahl >= 0)
                    {
                        f = multiplikation(syndrom[1], invFeld[xHyR(2, Hochzahl)], false);
                    }
                    kommentar = kommentar + "\nNur ein Fehler!\nHochzahl = " + Hochzahl.ToString() + ";   Fehlerwert = " + f.ToString() + "\n";
                    ausnahme1Fb2KB = true;
                    ausnahme1Fb2KB_stelle = Hochzahl;
                    ausnahme1Fb2KB_wert = f;
                }
                
            }
        }

        private void sigmapolynom_Auswertung_Click()
        {
            sigma[0] = 1;
            int k = codeWortlänge;
            int f = 0;
            kommentar = kommentar + "\n\nSigmapolynom_Auswertung\nSigmapolynom:\n";
            for (int i = 0; i < sigma.Length; i++)
            {
                kommentar = kommentar + sigma[i].ToString().PadLeft(3) + "*x^" + i.ToString().PadLeft(2);
                if (i< sigma.Length -1) kommentar = kommentar + " + ";
            }
            kommentar = kommentar + "\n\nFehlerstellen bei";

            for (int i = 0; i <= k - 1; i++)
            {
                int x = invFeld[xHyR(2, i)];
                
                int h = auswertung( sigma, x);
                kommentar = kommentar + "\n" + i.ToString().PadLeft(4) + "    " + x.ToString().PadLeft(4) + "   " + h.ToString().PadLeft(4);
                if (h == 0)
                {
                    fehlerstelle[f] = i;
                    kommentar = kommentar + " ---> " + i.ToString().PadLeft(3) + " ";
                    f++;
                }
                else kommentar = kommentar + "\n";
            }

            kommentar = kommentar + "\n\n";
        }


        private void fehlervektor_Ende_Click()
        {
            int w1 = codeWortlänge; // Anzahl der Lesebytes
            endeCode = new int[lesecode.Length];
            int[] fehlerpoly = new int[w1];

            if (fehlerAnzahl > 0)
            {
                if (ausnahme1Fb2KB == true)
                {
                    for (int i = 0; i < w1; i++) fehlerpoly[i] = 0;
                    fehlerpoly[ausnahme1Fb2KB_stelle] = ausnahme1Fb2KB_wert;

                    kommentar = kommentar + "Stelle: " +
                            ausnahme1Fb2KB_stelle.ToString().PadLeft(4) +
                            " mit Fehlerwert " + ausnahme1Fb2KB_wert.ToString().PadLeft(4) +
                            " --> " + (lesecode[ausnahme1Fb2KB_stelle] ^ ausnahme1Fb2KB_wert) + "\n";
                }
                else
                {
                    int[,] mat2 = new int[fehlerAnzahl, fehlerAnzahl];
                    for (int ze = 0; ze < fehlerAnzahl; ze++)
                        for (int sp = 0; sp < fehlerAnzahl; sp++)
                            mat2[ze, sp] = xHyR(2, (ze + 1) * fehlerstelle[sp]);
                    int[] ko2 = new int[fehlerAnzahl];
                    //for (int ze = 0; ze < fehlerAnzahl; ze++)
                    //    richTextBox14.AppendText(" " + syndrom[ze+1].ToString() + " ");

                    for (int ze = 0; ze < fehlerAnzahl; ze++) ko2[ze] = syndrom[ze + 1];
                    int[] f2 = new int[fehlerAnzahl + 1];
                    kommentar = kommentar + "\nFehlerwerte-LGS\n\n";
                    for (int ze = 0; ze < fehlerAnzahl; ze++)
                    {
                        kommentar = kommentar + " ";
                        for (int sp = 0; sp < fehlerAnzahl; sp++)
                        {
                            kommentar = kommentar + mat2[ze, sp].ToString().PadLeft(3) + " ";
                        }
                        kommentar = kommentar + " - " + ko2[ze].ToString().PadLeft(3) + "\n";
                    }
                    kommentar = kommentar + "\n\n";

                    f2 = LGS(mat2, ko2);
                    kommentar = kommentar + "\n\n";

                    for (int i = 0; i < w1; i++) fehlerpoly[i] = 0;
                    for (int i = 0; i < f2.Length - 1; i++)
                    {
                        fehlerpoly[fehlerstelle[i]] = f2[i + 1];
                        kommentar = kommentar + "Stelle: " + 
                            fehlerstelle[i].ToString().PadLeft(4) + 
                            " mit Fehlerwert " + f2[i + 1].ToString().PadLeft(4) +
                            " / und falschem Lesewert " + lesecode[fehlerstelle[i]].ToString().PadLeft(4) + 
                            " ----> " + (lesecode[fehlerstelle[i]] ^ fehlerpoly[fehlerstelle[i]])  + "\n";
                    }
                }
                for (int i = 0; i < w1; i++) endeCode[i] = lesecode[i] ^ fehlerpoly[i];
            }
            else
            {
                for (int i = 0; i < w1; i++) fehlerpoly[i] = 0;
                //endeCode = lesecode;
                for (int i = 0; i < w1; i++) endeCode[i] = lesecode[i];
            }

            int w2 = codeWortlänge - korrekturWortLänge;
            string endtext = "";

            for (int i = w1 - 1; i >= 0; i--)
            {
                
                if (i >= w1 - w2)
                {
                    if ((endeCode[i] - 1) > 31) endtext = endtext + (char)(endeCode[i] - 1);
                    else endtext = endtext + "[" + (endeCode[i]).ToString() + "]";
                }
                
            }
            ausnahme1Fb2KB = false;
        }


        public int[] RSDecode(string datenBytes, int anzahlECC)
        {
            kommentar = "";
            bildeArrayLesen_Click(datenBytes, anzahlECC);
            
            syndromB_Click();
            if (fehlerhaft == true)
            {
                determinaten_Click();
                     kommentar = kommentar + "\n1. Syndrom - LGS:\n";
                LGS_sigma_Click();
                if (mat.GetLength(0) > 1) sigmapolynom_Auswertung_Click();
            }
            fehlervektor_Ende_Click();

            int[] Hilf = new int[endeCode.Length];
            for (int i = 0; i < endeCode.Length; i++) Hilf[endeCode.Length - 1 - i] = endeCode[i];
            //for (int i = 0; i < endeCode.Length; i++) endeCode[i] = aHilf[i];
            return Hilf;
        }

        
    }
}
